/* Code to drive HD44780 Display
 * via PCF8574 I2C 8bit I/O Expander
 *
 * Version 1.0
 *
 * (c) Michael "ScriptKiller" Arndt
 * http://scriptkiller.de/
 * <scriptkiller@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */

//#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

/* include i2c.h from avrlib,
 * keep in mind that you have to link avrlib's i2c.c too!
 */
#include "i2c.h"

/* configuration is done in lcd.h */
#include "lcd.h"
#include "timer.h"
/*
void lcd_usleep(unsigned int msec) {

	//16-bit count, 4 cycles/loop 
_delay_loop_2(1000*msec);

}
*/
/* output (raw) data to lcd */
void lcdOut(unsigned char c) {
	i2cMasterSendNI(LCD_I2C_ADDR, 1, &c);
}


/* output data in 4bit mode to lcd
 * if data != 0, register select is set high!
 */
void lcd4BitOut(unsigned char c, unsigned char data) {

	unsigned char out;
	unsigned char rs;


	rs=0;
	if(data)
		rs=LCD_RS;

  /* 4 upper bits */
	out=0;
	if(c & 0x80) out |= LCD_D7;
	if(c & 0x40) out |= LCD_D6;
	if(c & 0x20) out |= LCD_D5;
	if(c & 0x10) out |= LCD_D4;
	lcdOut(out | rs);
  //  lcd_usleep(1);
	lcdOut(out | rs | LCD_E);
	timerPause(1);
	lcdOut(out | rs);

  /* 4 lower bits */
	out=0;
	if(c & 0x08) out |= LCD_D7;
	if(c & 0x04) out |= LCD_D6;
	if(c & 0x02) out |= LCD_D5;
	if(c & 0x01) out |= LCD_D4;
	lcdOut(out | rs);
  //  lcd_usleep(1);
	lcdOut(out | rs | LCD_E);
	timerPause(1);
	lcdOut(out | rs);


}


/* clear LCD */
void lcdClear() {

	lcd4BitOut(0x01, 0); // clear display

}

/* init lcd */
void lcdInit() {

	lcdOut(0x00);

  // A1
	lcdOut(LCD_D4 | LCD_D5);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5 | LCD_E);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5);

	timerPause(5);

  // A2
	lcdOut(LCD_D4 | LCD_D5);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5 | LCD_E);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5);

	timerPause(2);

  // A3
	lcdOut(LCD_D4 | LCD_D5);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5 | LCD_E);
	timerPause(10);
	lcdOut(LCD_D4 | LCD_D5);

	timerPause(10);

  // A4
  /* set mode to 4 bit */
	lcdOut(LCD_D5);
	timerPause(10);
	lcdOut(LCD_D5 | LCD_E);
	timerPause(10);
	lcdOut(LCD_D5);
	timerPause(10);

  /* from now on display can be interfaced
   * using 4bit mode */
	lcd4BitOut(0x28, 0); // A5
	lcd4BitOut(0x08, 0); // A6
	lcd4BitOut(0x01, 0); // A7
	lcd4BitOut(0x07, 0); // A8
	lcd4BitOut(0x0C, 0); // display + cursor off + blink off
	lcd4BitOut(0x06, 0); // increment addr

	lcdClear();

}


void lcdPrintString(unsigned char *str) {

	//	unsigned char c;
	//	unsigned char i;

	//	i=0;
	while( (*str) != '\0' ) {

//    if(i==LCD_WIDTH) {
      /* set DDRAM Addr 0x40 (beginning of 2nd line) */
		//		lcd4BitOut(0x80 | 0x40, 0);
		//	}
			//	str++;
	//	i++;

		lcd4BitOut((*str++), 1);

	}

}
void lcdPrintProgStr(const prog_char str[])
{
	// print a string stored in program memory
	register char c;

	// check to make sure we have a good pointer
	if (!str) return;

	// print the string until the null-terminator
	while((c = pgm_read_byte(str++)))
		lcd4BitOut(c,1);
}

void lcdPrintEEStr(const void *str)
{
	// print a string stored in program memory
	register char c;

	// check to make sure we have a good pointer
	if (!str) return;

	// print the string until the null-terminator
	while((c = eeprom_read_byte(str++)))
		lcd4BitOut(c,1);
}
