/* Code to drive HD44780 Display
 * via PCF8574 I2C 8bit I/O Expander
 *
 * (c) Michael "ScriptKiller" Arndt
 * http://scriptkiller.de/
 * <scriptkiller@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __LCD_H__
#define __LCD_H__

#include <avr/pgmspace.h>

#include "switchbox_i2c.h"

/* pin definitions */
#define P0 0x01
#define P1 0x02
#define P2 0x04
#define P3 0x08
#define P4 0x10
#define P5 0x20
#define P6 0x40
#define P7 0x80

/* define what display-pin is connected
 * to what io-pin of expander */
#define LCD_D4 P7
#define LCD_D5 P6
#define LCD_D6 P5
#define LCD_D7 P4

#define LCD_RS P1
#define LCD_RW P2
#define LCD_E  P3

/* what is the width of the display
 * (height is assumed to be 2) */
#define LCD_WIDTH 24

#define LCD_I2C_ADDR 	I2C_LCD
/* init the lcd */
void lcdInit(void);

/* clear the display */
void lcdClear(void);

/* print a string on display,
 * continues automatically in 2nd line,
 * note that lcdClear() should be called
 * before using this function */
void lcdPrintString(unsigned char *str);

void lcdPrintProgStr(const prog_char str[]);

void lcdPrintEEStr(const void *str);

/* low level function, see lcd.c */
void lcd4BitOut(unsigned char c, unsigned char data);

#endif
